from optparse import OptionParser
from types import ListType
import ConfigParser
import os
import re
import sys
import xmlrpclib

'''
Save a file like this in:

~/.zerp_config


----[ snip start ]-----------------------

[server]
connection: localhost

[server localhost]
username: user
password: pass
database: my-database-foo
rpc_url: http://localhost:8069/xmlrpc

[server localhost2]
username: user
password: pass
database: my-database-foo2
rpc_url: http://localhost:8069/xmlrpc


[server live]
prompt: This is the live server!
username: user
password: pass
database: live
rpc_url: https://liveserver:8069/xmlrpc

----[ snip end ]-----------------------

And choose which server is active via the "connection" parameter
'''

class client(object):
    def __init__(self,
                 connection=None,
                 config=None,
                 no_getopts=False,
                 getopts=None,
                 getopt_additional_options=None,
                 trim_fetches=False):
        """ Read configuration and open RPC connections 

        Alternative configuration can be loaded with the "connection" argument
        eg. client(connection="live")

        Using a completely different config altogether can also be achieved by using 
        the "config" argument.
        eg. client(keyword={
                      username: user
                      password: pass
                      database: my-database-foo2
                      rpc_url: http://localhost:8069/xmlrpc
                    })

        Any scripts that are currently using this client can take advantage of the
        commandline options to choose between server profiles. 

        eg. script.py -h

        eg. script.py -c localhost

        Note that any parameter based calls will override any commandline options

        To add functionality to getopt such as information on what the script does, 
        instantiate the client class with parameters such as "getopts" and/or 
        "getopt_additional_options".

        eg. c = client(getopts={
                          'usage':"usage: %prog [options] DATABASE1 DATABASE2 ..."
                      })

        The above example will produce a message like the following if the script 
        is called with -h:

            Usage: migrate.py [options] DATABASE1 DATABASE2 ...

            Options:
              -h, --help            show this help message and exit
              -l, --list            List available connections
              -c CONNECTION, --connection=CONNECTION
                                    Which connection to use

        Using "getopt_additional_options" will rack additional options as described
        here: http://docs.python.org/library/optparse.html#tutorial

        Set "no_getopts" to true to avoid any invocation of the getopt subroutines.
        
        Set trim_fetches to True to return raw ids instead of (id, name) tuples
        on many2one fields.
        """

        if not no_getopts:
            self.parse_options(getopts,getopt_additional_options)

        if config == None:
            if connection == None and self.options.connection:
                connection = self.options.connection
            config = self.load_config(connection)

        username = config.get('username') #the user
        self.pwd = config.get('password') #the password of the user
        self.dbname = config.get('database') #the database
        rpc_url = config.get('rpc_url')
        self.rpc_url = rpc_url
        self.trim_fetches = trim_fetches

        # Get the user_id
        sock_common = xmlrpclib.ServerProxy(rpc_url+'/common')
        self.user_id = sock_common.login(self.dbname, username, self.pwd)
        self.sock = xmlrpclib.ServerProxy(rpc_url+'/object')

        prompt = config.get('prompt')
        if prompt:
            print prompt
            print 'Type Enter to continue or Ctrl-C to abort.'
            sys.stdin.readline()

    def parse_options(self,getopts=None,getopt_additional_options=None):
        """ Subroutine to parse the commandline for options such as
            the selection of which server profile to use.
        """
        if getopts == None: getopts = {}
        parser = OptionParser(**getopts)

        parser.add_option(
            "-l", "--list", action="store_true", dest="list_connections",
            help="List available connections")

        parser.add_option(
            "-c", "--connection", dest="connection",
            help="Which connection to use")

        if getopt_additional_options:
            for options in getopt_additional_options:
                parser.add_option(*options)

        (self.options, self.args) = parser.parse_args()

        if self.options.list_connections:
            config = ConfigParser.ConfigParser()
            config.read(os.path.expanduser('~/.zerp_config'))
            default = config.get('server','connection')
            print "Groups in current configuration:\n"
            for section_name in sorted(config.sections()):
                match = re.search( "^server (.+)", section_name )
                if not match: continue
                profile_name = match.group(1)
                print "  ", "*" if default == profile_name else "-",
                print match.group(1)
            print "\nprofile marked with '*' is the current default"
            print
            sys.exit()

    def load_config(self,connection=None):
        defaults = {'prompt': ''}
        config = ConfigParser.ConfigParser(defaults)
        config.read(os.path.expanduser('~/.zerp_config'))
        server_conn_key = "server " + ( 
                              connection or config.get('server','connection')
                          )
        config = dict(config.items(server_conn_key))
        return config


    def search(self,
               model, 
               constraints, 
               span_start=0, 
               span_limit=100000, 
               order_by='',
               context=None):
        """ Get a list of ids for records that match the constraints.
        
        product_ids = client.search(
            'product.product', 
            ['|', ('name', 'ilike', 'base assy'),
             ('name', 'ilike', 'LSM Motor Assy')])
        """
        
        if context is None:
            context = {}
        context.setdefault('lang', u'en_CA')
        context.setdefault('tz', False)
        
        search_ids = self.execute(
            model,
            'search', 
            constraints, 
            span_start, 
            span_limit, 
            order_by, 
            context)
        return search_ids

    def fetch(self, model, ids, fields):
        """ Fetch fields for a specific set of records.
        
        client.fetch('stock.location', [1, 2, 3, 4], ['name'])
        
        If trim_fetches is true, then many2one fields will just have raw ids.
        """
        
        records = self.execute(
            model, 
            'read',
            ids,
            fields)
        if self.trim_fetches:
            for record in records:
                for name, value in record.iteritems():
                    if isinstance(value, ListType):
                        record[name] = value[0]
        return records or []

    def search_fetch(
        self,
        model, 
        constraints, 
        fields, 
        span_start=0, 
        span_limit=100000, 
        order_by='',
        context=None):
        """ Combine search and fetch in one step.
        
        product_names = client.search(
            'product.product', 
            ['|', ('name', 'ilike', 'base assy'),
             ('name', 'ilike', 'LSM Motor Assy')],
            ['name'])
        """
        
        search_ids = self.search(
            model, 
            constraints, 
            span_start, 
            span_limit, 
            order_by,
            context)
        if len(search_ids) == 0:
            return []
        return self.fetch(model, search_ids, fields)

    def update(self, model, ids, to_set):
        """ Update records.
        
        client.update(
            'stock.location', 
            [11], 
            {'name': 'doomed location'})
        """
        
        res = self.execute(
            model, 
            'write',
            ids,
            to_set)
        return res

    def create(self, model, fields):
        """ Create a new record.
        
        Returns the new record's id.
        
        client.create(
            'stock.location.path', 
            {
                'product_id': 1234,
                'location_from_id': 11,
                'location_dest_id': 23,
                'auto': 'transparent'
            })
        """
        rec_id = self.execute(
            model,
            'create',
            fields)
        return rec_id
    
    def unlink(self, model, ids):
        """ Delete a set of records.
        
        client.unlink('stock.move', [1, 2, 3, 4])
        """
        
        return self.execute(
            model,
            'unlink',
            ids)

    def execute(self, model, method, *args):
        """ Lets you execute any method on an object.
        
        For example, this will check availability on packing list id 1234:
        client.execute('stock.picking', 'action_assign', [1234])
        """
        return self.sock.execute(
            self.dbname,
            self.user_id,
            self.pwd,
            model,
            method,
            *args)

    def exec_workflow(self, model, signal, res_id):
        """ Lets you trigger the workflow on a record.
        
        For example, this will cancel purchase order 1234:
        client.exec_workflow('purchase.order', 'purchase_cancel', 1234)
        """
        return self.sock.exec_workflow(
            self.dbname,
            self.user_id,
            self.pwd,
            model,
            signal,
            res_id)



