#!/usr/bin/python

""" Copy several families of actuator products to new versions with the new
connector style.
"""

import re

from client import client

# category name: [product codes]
exclusions = {'NA': ['0049', '0107', '0652'],
              'NM': ['0049', '0107', '0652'],
              'LMG': ['0049', '0107'],
              'LSM': ['Set, LSM common components'],
              'G-LSQ': []}

def get_addition(category_name):
    """ Get the product code that should be added to BOM's for this category.
    """
    
    if category_name == 'LSM':
        return 'Set, LSM-T4 common components'
    return 'Assembly, T4 Connector'

class Product(object):
    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)
    
    def update_attrs(self, **kwargs):
        self.__dict__.update(kwargs)
        
class Bom(object):
    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)
        self.components = []
    
c = client(trim_fetches=True)

warehouse_rows = c.search_fetch('stock.warehouse',
                                [],
                                ['id', 'lot_stock_id'])
assert len(warehouse_rows) == 1, 'Only a single warehouse is supported.'
warehouse_id = warehouse_rows[0]['id']
stock_location_id = warehouse_rows[0]['lot_stock_id']

uom_rows = c.search_fetch('product.uom',
                          [('name', '=', 'PCE')],
                          ['id'])
assert len(uom_rows) == 1, 'Unit of measure name PCE must be unique.'
pieces_uom_id = uom_rows[0]['id']

# category id: [product ids]
exclusion_ids = {}
# category id: product id
addition_ids = {}
for row in c.search_fetch('product.category',
                          [('name', 'in', exclusions.keys())],
                          ['id', 'name']):
    category_id = row['id']
    name = row['name']
    product_codes = exclusions[name]
    product_rows = c.search_fetch('product.product',
                                  [('default_code', 'in', product_codes)],
                                  ['id'])
    product_ids = [product_row['id'] for product_row in product_rows]
    exclusion_ids[category_id] = product_ids
    added_product_code = get_addition(name)
    product_rows = c.search_fetch('product.product',
                                  [('default_code', '=', added_product_code)],
                                  ['id'])
    assert len(product_rows) == 1, "Product '%s' must exist." % added_product_code
    addition_ids[category_id] = product_rows[0]['id']
    
template_rows = c.search_fetch(
    'product.template',
    ['!',
     ('name', 'ilike', 'offline product'),
     ('categ_id', 'in', exclusion_ids.keys())],
    ['id', 'name', 'state', 'categ_id', 'description', 'description_purchase', 
     'description_sale', 'document_id', 'is_kit', 'lead_time_qty', 'list_price',
     'loc_case', 'loc_rack', 'loc_row', 'procure_method', 'produce_delay',
     'product_manager', 'purchase_ok', 'sale_delay', 'sale_ok', 
     'standard_price', 'supply_method', 'type', 'uom_id', 'uom_po_id', 
     'uos_coeff', 'uos_id', 'volume', 'warranty', 'weight', 'weight_net'])

templates = dict([(row['id'], Product(**row)) for row in template_rows])

product_rows = c.search_fetch(
    'product.product',
    [('product_tmpl_id', 'in', templates.keys())],
    ['id', 'product_tmpl_id', 'default_code', 'daily_sales_mode', 'daily_sales', 
     'notes', 'pricelist_purchase', 'pricelist_sale', 'purchase_line_warn',
     'purchase_line_warn_msg', 'safety_days', 'sale_line_warn', 
     'sale_line_warn_msg'])

products = {}
for row in product_rows:
    template_id = row['product_tmpl_id']
    product = templates[template_id]
    product.update_attrs(**row)
    if re.match('.*-ENG\d+$', product.default_code):
        continue
    if re.match('.*-M\d+(T2)?$', product.default_code):
        continue
    if re.match('.*-T\d+$', product.default_code):
        continue
    
    product.product_tmpl_id = template_id
    product.default_code += '-T4'
    if product.weight != 0:
        product.weight += 0.01
        product.weight_net += 0.01
    products[product.id] = product

print '\n'.join(sorted([product.default_code for product in products.values()]))
    
# find the parent BOM's for the products we are cloning.
boms = {}
product_boms = {}
for row in c.search_fetch('mrp.bom',
                          [('product_id', 'in', products.keys()),
                           ('bom_id', '=', False)],
                          ['id', 'autocomplete', 'code', 'name', 'product_id', 
                           'product_qty', 'product_rounding', 'product_uom',
                           'product_uos', 'product_uos_qty', 'routing_id',
                           'sequence', 'type']):
    bom = Bom(**row)
    bom.name += '-T4'
    boms[bom.id] = bom
    product_boms[bom.product_id] = bom
    
# Now find the BOM lines for the BOM's we just loaded.
for row in c.search_fetch('mrp.bom',
                          [('bom_id', 'in', boms.keys())],
                          ['bom_id', 'autocomplete', 'code', 'name', 
                           'product_id', 'product_qty', 'product_rounding', 
                           'product_uom',
                           'product_uos', 'product_uos_qty', 'routing_id',
                           'sequence', 'type']):
    bom = Bom(**row)
    parent_bom = boms[bom.bom_id]
    product = products[parent_bom.product_id]
    excluded_product_ids = exclusion_ids[product.categ_id]
    if bom.product_id not in excluded_product_ids:
        parent_bom.components.append(bom)
    
c.update('product.template', 
         [product.product_tmpl_id for product in products.itervalues()], 
         {'state': 'end'})
c.update('product.product', 
         products.keys(), 
         {'daily_sales': 0,
          'sale_line_warn': 'warning',
          'sale_line_warn_msg': ('Confirm that the customer actually wants ' +
                                 'the old T-MCA compatible version of this ' +
                                 'product.')})

for product in products.itervalues():
    template_id = c.create('product.template', 
                           {'name': product.name,
                            'state': product.state,
                            'categ_id': product.categ_id,
                            'description': product.description,
                            'description_purchase': product.description_purchase,
                            'description_sale': product.description_sale,
                            'document_id': product.document_id, 
                            'is_kit': product.is_kit,
                            'lead_time_qty': product.lead_time_qty,
                            'list_price': product.list_price,
                            'loc_case': product.loc_case,
                            'loc_rack': product.loc_rack,
                            'loc_row': product.loc_row,
                            'procure_method': product.procure_method,
                            'produce_delay': product.produce_delay,
                            'product_manager': product.product_manager, 
                            'purchase_ok': product.purchase_ok,
                            'sale_delay': product.sale_delay,
                            'sale_ok': product.sale_ok, 
                            'standard_price': product.standard_price,
                            'supply_method': product.supply_method,
                            'type': product.type,
                            'uom_id': product.uom_id,
                            'uom_po_id': product.uom_po_id,
                            'uos_coeff': product.uos_coeff,
                            'uos_id': product.uos_id,
                            'volume': product.volume,
                            'warranty': product.warranty,
                            'weight': product.weight,
                            'weight_net': product.weight_net})
    product_id = c.create('product.product',
                          {'product_tmpl_id': template_id,
                           'default_code': product.default_code,
                           'daily_sales_mode': product.daily_sales_mode,
                           'daily_sales': product.daily_sales,
                           'notes': product.notes,
                           'pricelist_purchase': product.pricelist_purchase,
                           'pricelist_sale': product.pricelist_sale,
                           'purchase_line_warn': product.purchase_line_warn,
                           'purchase_line_warn_msg': product.purchase_line_warn_msg,
                           'safety_days': product.safety_days,
                           'sale_line_warn': product.sale_line_warn,
                           'sale_line_warn_msg': product.sale_line_warn_msg})
    c.create('stock.warehouse.orderpoint',
             {'location_id': stock_location_id,
              'product_id': product_id,
              'product_max_qty': 0,
              'product_min_qty': 0,
              'product_uom': product.uom_id,
              'qty_multiple': 1,
              'warehouse_id': warehouse_id})
    bom = product_boms[product.id]
    bom_id = c.create('mrp.bom',
                      {'autocomplete': bom.autocomplete, 
                       'code': bom.code,
                       'name': bom.name,
                       'product_id': product_id, 
                       'product_qty': bom.product_qty, 
                       'product_rounding': bom.product_rounding, 
                       'product_uom': bom.product_uom,
                       'product_uos': bom.product_uos, 
                       'product_uos_qty': bom.product_uos_qty, 
                       'routing_id': bom.routing_id,
                       'sequence': bom.sequence,
                       'type': bom.type})
    for bom_line in bom.components:
        c.create('mrp.bom',
                 {'bom_id': bom_id,
                  'autocomplete': bom_line.autocomplete, 
                  'code': bom_line.code,
                  'name': bom_line.name,
                  'product_id': bom_line.product_id, 
                  'product_qty': bom_line.product_qty, 
                  'product_rounding': bom_line.product_rounding, 
                  'product_uom': bom_line.product_uom,
                  'product_uos': bom_line.product_uos, 
                  'product_uos_qty': bom_line.product_uos_qty, 
                  'routing_id': bom_line.routing_id,
                  'sequence': bom_line.sequence,
                  'type': bom_line.type})
    added_component_id = addition_ids[product.categ_id]
    c.create('mrp.bom',
             {'bom_id': bom_id,
              'name': 'T4',
              'product_id': added_component_id, 
              'product_qty': 1, 
              'product_uom': pieces_uom_id,
#              'product_rounding': bom_line.product_rounding, 
#              'product_uos': bom_line.product_uos, 
#              'product_uos_qty': bom_line.product_uos_qty, 
#              'routing_id': bom_line.routing_id,
#              'sequence': bom_line.sequence,
#              'type': bom_line.type
              })
   
#    print "[%s] %s" % (product.default_code, product.name)

print '%d products.' % len(products)
