#!/usr/bin/python

""" Find procurements that are still active, even though the purchase order
is not.

If the stock move is done, then check that the procurement is still running.

If the stock move is confirmed, check that it has no previous moves, and
call action_cancel().

If the procurement is still running, then check that the procurement workflow
is broken and set the procurement state to 'cancel'. Otherwise, trigger the
work flow with 'button_restart'.
"""

from client import client

c = client(trim_fetches=True)

broken_workflow_procurement_ids = []
move_ids = []
disconnected_move_ids = []
move2procurement = {}
procurement2move = {}
unexplained_procurement_ids = []

for procurement in c.search_fetch('mrp.procurement',
                                  [('state', 'in', ('running', 'ready')),
                                   ('purchase_id.state', 'in', ('cancel', 'done'))],
                                  ['id', 'state', 'move_id']):
    if procurement['state'] == 'running':
        broken_workflow_procurement_ids.append(procurement['id'])
        move_ids.append(procurement['move_id'])
    elif procurement['state'] == 'ready':
        move_ids.append(procurement['move_id'])
    else:
        print 'Unexplained procurement %d' % procurement['id']
    move2procurement[procurement['move_id']] = procurement['id']
    procurement2move[procurement['id']] = procurement['move_id']

if broken_workflow_procurement_ids:
    rows = c.search_fetch(
        'workflow.workitem',
        [('inst_id.res_type', '=', 'mrp.procurement'),
         ('inst_id.res_id', 'in', broken_workflow_procurement_ids)],
        ['inst_id'])
    instance_ids = [row['inst_id'] for row in rows]
    instances = c.fetch('workflow.instance',
                        instance_ids,
                        ['res_id'])
    unbroken_procurement_ids = [instance['res_id'] for instance in instances]
else:
    unbroken_procurement_ids = []

for proc_id in unbroken_procurement_ids:
    broken_workflow_procurement_ids.remove(proc_id)
    move_ids.remove(procurement2move[proc_id])
    print "Workflow not broken for procurement %d" % proc_id
    
c.update('mrp.procurement', 
         broken_workflow_procurement_ids, 
         {'state': 'cancel'})

print 'Canceled %d procurements.' % len(broken_workflow_procurement_ids)

for move in c.fetch('stock.move', move_ids, ['id', 'state']):
    proc_id = move2procurement[move['id']]
    if move['state'] == 'done':
        if proc_id not in broken_workflow_procurement_ids:
            print "Workflow not broken for procurement %d with 'done' move." % (
                proc_id)
    elif move['state'] == 'confirmed':
        disconnected_move_ids.append(move['id'])
    else:
        print "Unexpected move state '%s' for procurement %d." % (move['state'],
                                                                  proc_id)
    
if disconnected_move_ids:
    previous_moves = c.search_fetch('stock.move',
                                    [('move_dest_id', 'in', disconnected_move_ids)],
                                    ['id', 'state', 'move_dest_id'])
else:
    previous_moves = []

for move in previous_moves:
    print "Move not disconnected: %d" % move['move_dest_id']
    disconnected_move_ids.remove(move['move_dest_id'])

c.execute('stock.move', 'action_cancel', disconnected_move_ids)    

for move_id in disconnected_move_ids:
    procurement_id = move2procurement[move_id]
    c.exec_workflow('mrp.procurement', 'button_restart', procurement_id)

print 'Canceled %d moves and triggered work flow.' % len(disconnected_move_ids)
